function [gmmod,loglik]=gmmclass(X,Y,options)
% USAGE [svm]=svmclass2(X,Y,options)
%
% Gaussian Mixture Model for  CLASSIFICATION
% This routine classify the training set with a support vector machine
% using quadratic programming algorithm selected by options.solver
%
% INPUT
%
% Training set
%      X  		: input data
%      Y  		: output data
% parameters
%		options.nbgauss		: Bound on the lagrangian multipliers
%		options.nbiter		: nbiter (for EM)
%       options.solver : svm solver used (see svmclass_chooser)
%                           0 : EM default
%
% 		options.verbose : display outputs (default value is 0: no display)
%
%
%
%
%
% OUTPUT
%
% gmm.mix	mixtrures
% gmm.nbclass		bias
% 
%loglik loglikelyhood
%


%	02/24/2010 R. Flamary




if ~isfield(options,'nbgauss'); options.nbgauss=1; end
if ~isfield(options,'nbiterkmeans'); options.nbiterkmeans=5; end
if ~isfield(options,'nbiterem'); options.nbiterem=10; end
if ~isfield(options,'vartype'); options.vartype=1; end
if ~isfield(options,'ppcadim'); options.ppcadim=1; end
if ~isfield(options,'verbose'); options.verbose=0; end
if ~isfield(options,'solver'); options.solver=0; end


if ~isfield(options,'multiclass'); options.multiclass=0; end

if length(unique(Y))>2
    options.multiclass=1;
end



nbclass=length(unique(Y));

if length(options.nbgauss)==1
    options.nbgauss=options.nbgauss*ones(nbclass,1);
end
vals=unique(Y);

vartypes = {'spherical', 'diag', 'full', 'ppca'};

switch options.solver
    
    
    otherwise
        
        for i=1:nbclass
            Input=X(Y==vals(i),:);
            
            % create the model
            mix{i} = gmm(size(X,2), options.nbgauss(i), vartypes{options.vartype},options.ppcadim);
            
            % initialize the model
            opts = foptions;
            opts(14) = options.nbiterkmeans;
            mix{i}= gmminit(mix{i}, Input, opts);
            
            % learning
            
            opts = zeros(1, 18);
            opts(1)  = options.verbose;		% Prints out error values.
             opts(14) =options.nbiterem ;		% Max. Number of iterations.

            [mix{i}, opts, errlog] = gmmem(mix{i}, Input, opts);
            
           
        end
    
end

gmmod.mix=mix;

gmmod.nbclass=nbclass;

loglik=0;












