function [y,ydec]=gmmval(x,gmm)

% USAGE
% [y,yval]=gmmval(x,gmm)
%
% svmval computes the prediction of a support vector machine
%       using the kernel function and its parameter for classification
%		  or regression
%
% INPUT
% x    : input data
% svm.xsup : support vector list
% svm.w    : weight
% svm.kernel : string containing the type of kernel
% svm.kerneloption : setting parameter of kernel.
% svm.w0   : bias. this can be a column vector in case of semiparametric SVM
% svm.span : span matrix for semiparametric SVM
%
%   ----- 1D Frame Kernel --------------------------
%
%   svm.framematrix  frame elements for frame kernel
%   svm.vector       sampling position of frame elements
%	 svm.dual 		  dual frame
%
% OUTPUT
%
% ydec : sign of the output ouf the network at point (vector or matrix) x
%
%       y = w phi(x) - b*span(x)
%
%
%	See also svmclass,svmreg, svmkernel
%
%

%	01/12/2009 R. Flamary



%
% Usual verifications
%
if gmm.nbclass>2
    multiclass=1;
else
    multiclass=0;
end

if multiclass==0
    
   y=gmmprob(gmm.mix{2}, x)-gmmprob(gmm.mix{1}, x);
    ydec=sign(y);

else

    y=zeros(size(x,1),gmm.nbclass);



            for i=1:gmm.nbclass

                y(:,i)=gmmprob(gmm.mix{i}, x);


            end

            [temp,ydec]=max(y,[],2);

    end



end
