function [xmin,vars,costmin,LOG]=grad_desc(x0,vars,data,opts)
% [x,cost,vars]=grad_desc(x0,vars0,data,opts)
% function doing a line_search (golden) using the cost function and
% gradient function defined in opts.
% input:
%   x0 : value of x before the descent (x is a column vector)
%   vars0: stucture needed by the cost function and the gradient function
%          (this structure can be updated for warm start or not )
%   data : constant data used in the cost and gradient function
%   opts : options fior the gradient descent
%   opts.costfunc: function for the cost that is used like that :
%           [costnew,varsnew]=costfunc(xnew,data,vars);
%   opts.gradfunc: function returning the gradient:
%           [gradf]=gradfunc(x0,data,vars0)
%   opts.use_cg : contains wether or not to use conjugate gradient
%                 if not zero then the number is the type of cg (see code)
%   opts.goldensearch_stepmax: the max step of the golden search
%   opts.goldensearch_deltmax: the max delta of the golden search
%   opts.verbose: 0 if not, 1 if print + nbspaces
%   opts.log : 1 if saving LOG
%   opts.stopvarx : x variation threshold for stoping 
%   opts.stopvarj : J cost variation threshold for stoping 
%   opts.nbitermax: max number of iterations
%   opts.numericalprecision : numerical precision for the minimization
%
%  Author : Remi Flamary remi (dot) flamary (at) gmail (dot) com

if ~isfield(opts,'use_cg'); opts.use_cg=0; end
if ~isfield(opts,'goldensearch_stepmax'); opts.goldensearch_stepmax=1; end
if ~isfield(opts,'goldensearch_deltmax'); opts.goldensearch_deltmax=0.01 ;end
if ~isfield(opts,'g_1'); opts.g_1=1; end
if ~isfield(opts,'d_1'); opts.d_1=0; end
if ~isfield(opts,'verbose'); opts.verbose=0; end
if ~isfield(opts,'log'); opts.log=1; end
if ~isfield(opts,'stopvarx'); opts.stopvarx=0; end
if ~isfield(opts,'stopvarj'); opts.stopvarj=1e-3; end
if ~isfield(opts,'nbitermax'); opts.nbitermax=100; end
if ~isfield(opts,'numericalprecision'); opts.numericalprecision=1e-5; end
if ~isfield(opts,'printfunc'); opts.printfunc=@print_0; end
%if ~isfield(opts,'projfunc'); opts.projfunc=@proj_0; end

LOG=[];
J=[];
time=[];


x=x0;


goldensearch_deltmaxinit= opts.goldensearch_deltmax;
goldensearch_stepmax=1e-8;

%initialisation
[J,vars]=opts.costfunc(x0,data,vars);

if opts.verbose~=0
    opts.printfunc(0,opts.verbose,J,0,vars);
end

if opts.log
    LOG.vars=vars;
end

nbverbose=0;
loop=1;
nbloop=1;

% MAIN LOOP

%tic;

  while loop
        %descente de gradient

        time(nbloop)=cputime;

        xold=x;

        [x,J(nbloop+1),vars,opts]=grad_desc_update(x,vars,data,J(nbloop),opts);


        % nétoyage numérique
        x((abs(x)<opts.numericalprecision))=0;


        if opts.verbose ~= 0
            % verbose print function
            nbverbose=opts.printfunc(nbverbose,opts.verbose,J(nbloop+1),(J(nbloop+1)-J(nbloop))/J(nbloop),vars);
        end


        if opts.log==1
       
        %   time(nbloop+1)=toc;
           xtot{nbloop+1}=x;  

        end


        if nbloop>opts.nbitermax
            loop=0;
            disp('Max number of iteration reached');
        end



        % J variation stopping criterion
            if J(nbloop+1)-J(nbloop)>0
                % loop=0;
                disp('DeltaJ>0');
            end
            if abs(J(nbloop+1)-J(nbloop))/J(nbloop)*sign(J(nbloop))<=opts.stopvarj
                loop=0;
                disp('DeltaJ convergence');
            end
            
        % x variation stopping criterion
       if max(max(max(abs(x-xold))))/sum(sum(sum(abs(x))))<opts.stopvarx
                  loop=0;
                disp('Delta x convergence');
       end

        % adaptative step
        if max(max(max(abs(x-xold))))/sum(sum(sum(abs(x))))<opts.numericalprecision*1000 && opts.goldensearch_deltmax > goldensearch_stepmax
            opts.goldensearch_deltmax=opts.goldensearch_deltmax/100;
            %  loop=1;
        elseif opts.goldensearch_deltmax~=goldensearch_deltmaxinit
            opts.goldensearch_deltmax=opts.goldensearch_deltmax*100;

        end;


        nbloop=nbloop+1;


  end

    xmin=x;
    costmin=J(end);
    
    if opts.log
        LOG.J=J;
        LOG.xtot=xtot;
        LOG.time=time;
        
    end
   
end

function [nbverbose]=print_0(nbverbose,verbose,J,deltaJ,vars)

nbverbose = nbverbose+1;
if nbverbose == 20 || (nbverbose==1 && deltaJ==0);
    disp([repmat('    ',1,verbose-1) '      Cost     Delta Cost ']);
    nbverbose = 0;
end


fprintf(1,[repmat('    ',1,verbose-1) '| %11.4e | %8.4f |\n'],[J deltaJ ]);

end
















