function [xmin,vars,costmin,LOG]=nesterov_solver(x0,vars,data,opts)
% [x,cost,vars]=iterative_solver(x0,vars0,data,opts)
% function doing function executing iteratively an update function until
% convergence conditions
% input:
%   x0 : value of x before the descent (x is a column vector)
%   vars0: stucture needed by the cost function and the gradient function
%          (this structure can be updated for warm start or not )
%   data : constant data used in the cost and gradient function
%   opts : options fior the gradient descent
%   opts.q_func: function for the cost that is used like that :
%           [costnew,varsnew]=q_func(W,Wt,vars,data);
%   opts.minq_func: function for the cost that is used like that :
%           [x,costnew,varsnew]=min_func(x_1,vars,data);
%   opts.costfunc: function for the cost that is used like that :
%           [costnew,varsnew]=costfunc(x_1,vars,data);
%   opts.verbose: 0 if not, 1 if print + nbspaces
%   opts.log : 1 if saving LOG
%   opts.stopvarx : x variation threshold for stoping 
%   opts.stopvarj : J cost variation threshold for stoping 
%   opts.nbitermax: max number of iterations
%   opts.numericalprecision : numerical precision for the minimization
%
%  Author : Remi Flamary remi (dot) flamary (at) gmail (dot) com

if ~isfield(opts,'verbose'); opts.verbose=0; end
if ~isfield(opts,'log'); opts.log=1; end
if ~isfield(opts,'stopvarx'); opts.stopvarx=0; end
if ~isfield(opts,'stopvarj'); opts.stopvarj=1e-3; end
if ~isfield(opts,'nbitermax'); opts.nbitermax=100; end
if ~isfield(opts,'numericalprecision'); opts.numericalprecision=1e-5; end
if ~isfield(opts,'printfunc'); opts.printfunc=@print_0; end
if ~isfield(opts,'nu'); opts.nu=1.1; end
if ~isfield(opts,'L0'); opts.L0=1; end


LOG=[];
J=[];
time=[];


W=x0;

% initialization
[J,vars]=opts.costfunc(W,vars,data);

if opts.verbose==1
    opts.printfunc(0,opts.verbose,J,0,vars);
end

if opts.log
    LOG.vars=vars;
end

nbverbose=0;
loop=1;
nbloop=1;

vars.L=opts.L0;
vars.V=W;
vars.a_1=1;
vars.t=0;
vars.dt=zeros(size(W));

% MAIN LOOP

%tic;
W_1=W;

  while loop

     loop2=1;
    % F=1;Q=0;
      while loop2
        %while (opts.costfunc(vars.V,vars,data)>opts.q_func(W,vars.V,vars,data)), vars.L =vars.L*opts.nu; end
        [W,ctemp,vars]=opts.minq_func(vars.V,vars,data);
        
        F=opts.costfunc(W,vars,data);
        Q=opts.q_func(W,vars.V,vars,data);
        
        if F<=Q
            loop2=0;
        else
            vars.L =vars.L*opts.nu;
        end
        
        
      end
      
       J(nbloop+1)=F;
       
       vars.a=2/(vars.t+3);
       
       vars.dt=W-W_1;
       
       vars.V=W+(1-vars.a)/vars.a*vars.a_1*vars.dt;
       
       %update
       vars.a_1=vars.a;
       W_1=W;
       vars.t=vars.t+1;
        
        %[W,J(nbloop+1),vars]=opts.updatefunc(W,vars,data);


        % nétoyage numérique
        W((abs(W)<opts.numericalprecision))=0;


        if opts.verbose ~= 0
        % verbose print function
            nbverbose=opts.printfunc(nbverbose,opts.verbose,J(nbloop+1),(J(nbloop+1)-J(nbloop))/J(nbloop),vars);
         end


        if opts.log==1
       
           %time(nbloop+1)=toc;
%           xtot{nbloop+1}=x;  

        end


        if nbloop>opts.nbitermax
            loop=0;
            disp('Max number of iteration reached');
        end



        % J variation stopping criterion
            if J(nbloop+1)-J(nbloop)>0
                % loop=0;
              %  disp('DeltaJ>0');
            end
            if abs(J(nbloop+1)-J(nbloop))/J(nbloop)<=opts.stopvarj
                loop=0;
                disp('DeltaJ convergence');
            end
            
        % x variation stopping criterion
       if max(max(max(abs(W-W_1))))/sum(sum(sum(abs(W))))<opts.stopvarx
                  loop=0;
                disp('Delta x convergence');
       end

  

        nbloop=nbloop+1;


  end

    xmin=W;
    costmin=J(end);
    
    if opts.log
        LOG.J=J;
%        LOG.xtot=xtot;
        LOG.time=time;
        
    end


end


function [nbverbose]=print_0(nbverbose,verbose,J,deltaJ,vars)

nbverbose = nbverbose+1;
if nbverbose == 20 || (nbverbose==1 && deltaJ==0);
    fprintf(1,[repmat('    ',1,verbose-1) '      Cost     Delta Cost \n']);
    nbverbose = 0;
end


fprintf(1,[repmat('    ',1,verbose-1) '| %11.4e | %8.4f |\n'],[J deltaJ ]);

end