function [pest]=pairwise(r,v_nb_indiv)
%
% USAGE [pest]=pairwise(r, v_nb_indiv)
%
% T.-K. Huang, R. C. Weng, and C.-J. Lin.
% Generalized Bradley-Terry Models and Multi-class Probability Estimates.
%  Journal of Machine Learning Research, 7(2006), 85-115.
%  A (very) short version of this paper appears in NIPS 2004. 
%
%  http://www.csie.ntu.edu.tw/~cjlin/libsvmtools/
% http://www.csie.ntu.edu.tw/~cjlin/libsvmtools/libsvm-errorcode/generalBT.zip
%

k = size(r,1);
pest = sum(r,2)*2/k/(k-1);

% maxiter=1000;
maxiter=5;
pest = pest / sum(pest);
mu=zeros(k,k);
for i =1:k
  mu(i,i+1:k) = pest(i) ./ (pest(i)+pest(i+1:k)') ;
  mu(i+1:k,i) = 1 - mu(i,i+1:k)';
end

for t=1:maxiter,
  for i = 1:k,
    alpha =  (r(i,:) * v_nb_indiv(:) )    /    (mu(i,:)* v_nb_indiv(:) );
    pest(i) = pest(i)*alpha;
    noti=[1:i-1,i+1:k];
    mu(i,noti) = alpha*mu(i,noti) ./ (alpha*mu(i,noti)+ mu(noti,i)');
    mu(noti,i) = 1 - mu(i,noti)';
  end
  pest = pest ./ sum(pest);
%   if (max(abs(sum(r,2)./sum(mu,2)-1)) < 0.001)
  if (max(abs(sum(r,2)./sum(mu,2)-1)) < 0.01)
    return;
  end
end
%fprintf(1, '\n Maximum iteration reached\n');
%fprintf(1, '.');

