function [svm,obj,LOG]=lgrouplearn(X,Y,options)
%  finding a linear mapping between X and Y
%
% linear svm for group norm
%   min  \sum_i loss(yi,x_i*w+w0)  + l1-l2(w)
%   w,w0
% INPUT
%
% Training set
%      X  		: input data
%      Y  		: output data
% parameters
%		options.lambda		: regularisation term
%       options.groups  : groups for the l1_l2 norm
%       options.loss    : 0 l2 regression
%                         1 squared hinge
%                         2 logistic loss
%
% OUTPUT
%
% svm.w      weight
% svm.w0		bias
% svm.pos    position of Support Vector
% obj    Value of Objective function
%
%
% see also svmreg, svmkernel, svmval, svmclass


%	01/12/2009 R. Flamary


if ~isfield(options,'lambda'); options.lambda=1; end

if ~isfield(options,'groups'); options.groups=1:size(X,2); end
if ~isfield(options,'multiclass'); options.multiclass=0; end

if length(unique(Y))>2
    options.multiclass=1;
end


if options.multiclass==0

    if nargin>3
        options.alphainit=alphainit;
    end

    %   options.C=options.C/length(Y);


    % ajout du biais 
    options.groups=[options.groups(:);max(max(options.groups(:)))+1];

    data=options;

    data.X=[X, ones(size(X,1),1)];
    data.Y=Y;


    %% initialisation


    data.options=options;

    opts=options;


    switch options.loss

        case 1

            if ~isfield(options,'W0');
                [w,w0] = lsvmclass(X,Y,1/options.lambda);

                W0=[w;w0];
            else
                W0=options.W0;
            end

            opts.q_func=@q_func_hinge;
            opts.minq_func=@minq_func_hinge;
            opts.costfunc=@costfunc_hinge;
            
        case 2      
            if ~isfield(options,'W0');
                Xt=[X,ones(size(X,1),1)];
                
               % W0=(Xt'*Xt+ options.lambda*eye(size(Xt,2)))\Xt'*Y;
               W0=rand(size(X,2)+1,1);
            else
                W0=options.W0;
            end

            opts.q_func=@q_func_log;
            opts.minq_func=@minq_func_log;
            opts.costfunc=@costfunc_log;

        otherwise
            
            if ~isfield(options,'W0');
                Xt=[X,ones(size(X,1),1)];
                W0=rand(size(X,2)+1,1);
              %  W0=(Xt'*Xt+ options.lambda*eye(size(Xt,2)))\Xt'*Y;
            else
                W0=options.W0;
            end

            opts.q_func=@q_func_reg;
            opts.minq_func=@minq_func_reg;
            opts.costfunc=@costfunc_reg;


    end

    [W,vars,obj,LOG]=nesterov_solver(W0,[],data,opts);

    svm.xsup=[];
    svm.w=W(1:end-1);
    svm.w0=W(end);
    svm.pos=[];
    svm.kerneloption=1;
    svm.kernel='linear';


else

    vals=sort(unique(Y),'ascend');

    if options.verbose~=0
        disp(['Multiclass SVM classification (' num2str(length(vals)) 'classes)' ]);
    end

    switch options.multiclass

        case 1 % one against all


            options2=options;
            options2.multiclass=0;

            for i=1:length(vals)


                if options.verbose~=0
                    disp(['Classification :' num2str(i) '/' num2str(length(vals)) ]);
                end;
                Yt=(Y==vals(i))-(Y~=vals(i));
                [models{i},obj(i),LOG{i}]=lgrouplearn(X,Yt,options2);

            end

            svm.multiclass=options.multiclass;
            svm.nbclass=length(vals);
            svm.models=models;
            svm.kerneloption=1;
            svm.kernel='linear';
            svm.vals=vals;




    end

end

end

function [costnew,varsnew]=q_func_hinge(W,Wt,vars,data)

varsnew=vars;

%V=W-gradf(W,vars,data)/vars.L;

fw=1/2*sum(max(0,1-data.Y.*(data.X*Wt)).^2)+data.lambda*Omega12_group(W,data.groups);

c2=sum( ( (W-Wt).*gradf_hinge(Wt,vars,data) )) + vars.L/2*sum((W-Wt).^2);

costnew=fw+c2;

end

function [costnew,varsnew]=costfunc_hinge(W,vars,data)

varsnew=vars;

costnew=1/2*sum(max(0,1-data.Y.*(data.X*W)).^2)+data.lambda*Omega12_group(W(1:end-1),data.groups(1:end-1));

end


function [W,obj,vars]=minq_func_hinge(Vt,vars,data)

lambda=data.lambda/vars.L;

W=zeros(size(Vt));

Vt=Vt-1/vars.L*gradf_hinge(Vt,vars,data);

vals=unique(data.groups);

for i=1:length(vals)
    if norm(Vt(data.groups==vals(i)))>lambda
        W(data.groups==i)=(1-lambda/norm(Vt(data.groups==vals(i)))).*Vt(data.groups==vals(i));
    end
end

W(end)=Vt(end);

obj=0;

end

function Df=gradf_hinge(W,vars,data)

Df=-sum(data.X.*repmat(data.Y.*max(0,1-data.Y.*(data.X*W)),1,size(data.X,2)))';
end



function [costnew,varsnew]=q_func_reg(W,Wt,vars,data)

varsnew=vars;

%V=W-gradf(W,vars,data)/vars.L;

fw=1/2*sum((data.Y-(data.X*Wt)).^2)+data.lambda*Omega12_group(W,data.groups);

c2=sum( ( (W-Wt).*gradf_reg(Wt,vars,data) )) + vars.L/2*sum((W-Wt).^2);

costnew=fw+c2;

end

function [costnew,varsnew]=costfunc_reg(W,vars,data)

varsnew=vars;

costnew=1/2*sum((data.Y-(data.X*W)).^2)+data.lambda*Omega12_group(W(1:end-1),data.groups(1:end-1));

end


function [W,obj,vars]=minq_func_reg(Vt,vars,data)

lambda=data.lambda/vars.L;

W=zeros(size(Vt));

Vt=Vt-1/vars.L*gradf_reg(Vt,vars,data);

vals=unique(data.groups);

for i=1:length(vals)
    if norm(Vt(data.groups==vals(i)))>lambda
        W(data.groups==i)=(1-lambda/norm(Vt(data.groups==vals(i)))).*Vt(data.groups==vals(i));
    end
end

W(end)=Vt(end);

obj=0;

end

function Df=gradf_reg(W,vars,data)

Df=data.X'*(data.X*W-data.Y);
end

function [costnew,varsnew]=q_func_log(W,Wt,vars,data)

varsnew=vars;

%V=W-gradf(W,vars,data)/vars.L;

fw=1/2*sum(log(1+exp(-data.Y.*(data.X*Wt))))+data.lambda*Omega12_group(W,data.groups);

c2=sum( ( (W-Wt).*gradf_log(Wt,vars,data) )) + vars.L/2*sum((W-Wt).^2);

costnew=fw+c2;

end

function [costnew,varsnew]=costfunc_log(W,vars,data)

varsnew=vars;

costnew=1/2*sum(log(1+exp(-data.Y.*(data.X*W))))+data.lambda*Omega12_group(W(1:end-1),data.groups(1:end-1));

end


function [W,obj,vars]=minq_func_log(Vt,vars,data)

lambda=data.lambda/vars.L;

W=zeros(size(Vt));

Vt=Vt-1/vars.L*gradf_log(Vt,vars,data);

vals=unique(data.groups);

for i=1:length(vals)
    if norm(Vt(data.groups==vals(i)))>lambda
        W(data.groups==i)=(1-lambda/norm(Vt(data.groups==vals(i)))).*Vt(data.groups==vals(i));
    end
end

W(end)=Vt(end);

obj=0;

end

function Df=gradf_log(W,vars,data)
temp=exp(-data.Y.*(data.X*W));
temp2=-data.Y.*temp./(1+temp);
Df=sum(data.X.*repmat(temp2,1,size(data.X,2)))';
end





