function [xsup,w,d,pos,timeps,alpha,obj]=svmclasssimple(x,y,c,lambda,kernelt,kerneloption,verbose,span, alphainit,options)
% USAGE [xsup,w,b,pos,timeps,alpha,obj]=svmclass(x,y,c,lambda,kernel,kerneloption,verbose,span, alphainit)
%
% Support vector machine for CLASSIFICATION
% This routine classify the training set with a support vector machine
% using quadratic programming algorithm (active constraints method)
%
% INPUT
%
% Training set
%      x  		: input data 
%      y  		: output data
% parameters
%		c		: Bound on the lagrangian multipliers     
%		lambda		: Conditioning parameter for QP method
%		kernel		: kernel  type. classical kernel are
%
%		Name			parameters
%		'poly'		polynomial degree
%		'gaussian'	gaussian standard deviation
%
%		for more details see svmkernel
% 
%		kerneloption : parameters of kernel
%
%		for more details see svmkernel
%
% 		verbose : display outputs (default value is 0: no display)
%
%     Span    : span matrix for semiparametric learning 
%               This vector is sized Nbapp*Nbbasisfunction where
%               phi(i,j)= f_j(x(i));
%
%       options.AdaptativeC: use a C adaptative with the number of points
%
%
%
% OUTPUT
%
% xsup	coordinates of the Support Vector
% w      weight
% b		bias
% pos    position of Support Vector
% timeps time for processing the scalar product
% alpha  Lagragian multiplier
% obj    Value of Objective function
%
%
% see also svmreg, svmkernel, svmval

%	21/09/97 S. Canu
%	04/06/00 A. Rakotomamonjy   -inclusion of other kernel functions
%	04/05/01 S. Canu            -inclusion of multi-constraint optimization for frame-SVM
%
%       scanu@insa-rouen.fr, alain.rakoto@insa-rouen.fr


if nargin<10
    options.AdaptativeC=0;
end

if nargin< 9
    alphainit=[];
end;

if nargin < 8 | isempty(span)
    A = y;
    b = 0;
else
    if span==1
        span=ones(size(y));
    end;
    [na,m]=size(span);
    [n un] = size(y);
    if n ~= na
        error('span, x and y  must have the same number of row')
    end
    A = (y*ones(1,m)).*span;
    b = zeros(m,1);
end
if nargin < 7
    verbose = 0;
end

if nargin < 6
    gamma = 1;
end

if nargin < 5
    kernelt = 'gaussian';
end

if nargin < 4
    lambda = 0.000000001;
end

if nargin < 3
    c = inf;
end


if options.AdaptativeC==1
    c=GetAdaptativeC(x,y,c);
end

[n un] = size(y);

if ~isempty(x)
    [nl nc] = size(x);
    if n ~= nl
        error('x and y must have the same number of row')
    end
end;

if min(y) ~= -1
    error(' y must coded: 1 for class one and -1 for class two')
end

%if verbose ~= 0 disp('building the distance matrix'); end;

global svModel;
global warmstart;

e = ones(size(y));
donnees = data(x',y,[],[]);
timeps = 0;

kernelt2=convertkerneltosimple(kernelt);

method='chol';
multiclass = '';
C = c;
tolerance = lambda;
noyau = kernel(kernelt2,kerneloption);
pas = 100;
parametres = param(C,pas,multiclass,method,tolerance);
parametres.verbose=verbose;

if isempty(alphainit)
    trainSVM(donnees, noyau, parametres);
    warmstart=svModel.model;
else
   trainSVM(donnees, noyau, partametres,warmstart);
end

alpha=svModel.model.alpha{1};
pos=svModel.model.indices{1};

alphaall=zeros(size(e));
alphaall(pos)=alpha;
%obj=-0.5*alphaall'*H*alphaall +e'*alphaall;
obj=0;
if ~isempty(x)
    xsup = x(pos,:);
else
    xsup=[];
end;

ysup = y(pos);



w = (alpha.*ysup);
d = svModel.model.mu{1};

end


function k=convertkerneltosimple(kerneltype)
% à compléter
k='gauss';

end


