function [svm,obj]=svmoneclass2(X,options,alphainit)
% USAGE [svm]=svmclass2(X,Y,options)
%
% Support vector machine for CLASSIFICATION
% This routine classify the training set with a support vector machine
% using quadratic programming algorithm selected by options.solver
%
% INPUT
%
% Training set
%      X  		: input data 
%      Y  		: output data
% parameters
%		options.C		: Bound on the lagrangian multipliers     
%		options.lambda		: Conditioning parameter for QP method
%       options.solver : svm solver used (see svmclass_chooser)
%                           0 : monqp.m
%                           1 : gradient descent (Chapelle)
%                           2 : SimpleSVM (Looslie)
%                           3 : svqp2 (Leon Bottou)
%                           4 : sparse svm ( Chapelle)
%
%		options.kernel		: kernel  type. classical kernel are
%
%		Name			parameters
%		'poly'		polynomial degree
%		'gaussian'	gaussian standard deviation
%
%		for more details see svmkernel
% 
%		options.kerneloption : parameters of kernel
%
%		for more details see svmkernel
%
% 		options.verbose : display outputs (default value is 0: no display)
%
%      options.Span    : span matrix for semiparametric learning 
%               This vector is sized Nbapp*Nbbasisfunction where
%               phi(i,j)= f_j(x(i));
%
%       
%
%
%
% OUTPUT
%
% svm.xsup	coordinates of the Support Vector
% svm.w      weight
% svm.w0		bias
% svm.pos    position of Support Vector
% obj    Value of Objective function
%
%
% see also svmreg, svmkernel, svmval, svmclass


%	01/12/2009 R. Flamary




if ~isfield(options,'lambda'); options.lambda=1e-10; end
if ~isfield(options,'kernel'); options.kernel='gaussian'; end
if ~isfield(options,'kerneloption'); options.kerneloption=1; end
if ~isfield(options,'nu'); options.nu=0.05; end
if ~isfield(options,'solver'); options.solver=0; end
if ~isfield(options,'span'); options.span=[]; end
if ~isfield(options,'verbose'); options.verbose=0; end
if ~isfield(options,'alphainit'); options.alphainit=[]; end

if nargin>3
    options.alphainit=alphainit;
end

%options.C=1/(length(Y)*options.nu);;

switch options.solver
    case 1
     [xsup,w,d,pos,obj]=svmoneclass(X,options.kernel,options.kerneloption,options.nu,options.verbose,options.alphainit);
   obj=0;
    case 2
[xsup,w,d,pos,obj]=svmoneclassvapnik(X,options.kernel,options.kerneloption,options.C,options.verbose,options.alphainit);

end
svm.xsup=xsup;
svm.w=w;
svm.w0=d;
svm.pos=pos;
svm.kerneloption=options.kerneloption;
svm.kernel=options.kernel;
svm.span=options.span;








