%lassification 
%   Comparing Gaussian and polynomial Kernels 
%
%


clear all
close all

%%
%
%   Creating Data
%
seed=1985
rand('state',seed);
randn('state',seed);

nbapp=10000;
nn=20;
nbtest=nn*nn;
sigma=0.2;
[xapp,yapp,xtest,ytest,xtest1,xtest2]=datasets('gaussian',nbapp,nbtest,sigma);
%------- Building a 2D Grid for function evaluation -------------------------
[xtest1 xtest2]  = meshgrid([-1:.05:1]*3.5,[-1:0.05:1]*3); 
nn = length(xtest1); 
xtest = [reshape(xtest1 ,nn*nn,1) reshape(xtest2 ,nn*nn,1)]; 

idxs=randperm(length(yapp));

yapp=yapp(idxs);
xapp=xapp(idxs,:);

nbnoise=1000;
pbruit=1;
xapp=[xapp pbruit*randn(size(xapp,1),nbnoise)];
xtest=[xtest pbruit*zeros(size(xtest,1),nbnoise)];

%
%  learning with polynomial kernels
%

%% test GMM

options.nbgauss=[12];
options.nbiterem=50;
options.nbiterkmeans=20;
options.verbose=1;
options.vartype=2;


tic 
[gmm,obj]=gmmclass(xapp,yapp,options)
toc

[ypred,ydec] = gmmval(xtest,gmm);
ypred = reshape(ydec,nn,nn); 


%% biclasse

options.lambda = 1e-10;  
options.C = 1e4;
options.kernel='gaussian';
options.kerneloption=1;
options.verbose=0;
options.solver=4;
options.multiclass=0;

tic 
[svm,obj]=svmclass2(xapp,yapp,options)
toc

[ypred] = svmval2(xtest,svm);
ypred = reshape(ypred,nn,nn); 

%% linear group svm

options.C = 1e5;

options.verbose= 0;%: print everything
options.log =1;%: log informations
options.nbitermax =100 ;%: log informatioouns
options.stopvarx=[0e0];% : threshold for stopping on F changes
options.stopvarj=[1e-8];
options.usegolden=1;
options.goldensearch_deltmax=.01;
options.goldensearch_stepmax=1;
options.numericalprecision=1e-3;

options.groups=[1:10]%ones(10,1)%ones(10,1)%1:10
options.nu=2;
options.L0=1;

tic 
[svm,obj,LOG]=lgroupsvmclass(xapp,yapp,options)
toc

[ypred] = svmval2(xtest,svm);
ypred = reshape(ypred,nn,nn); 

figure(1)
plot(svm.w, 'DisplayName', 'svm.w', 'YDataSource', 'svm.w'); figure(gcf)
figure(2)
plot(LOG.J)


%% linear group general

options.lambda = 5e1;

options.verbose= 1;%: print everything
options.log =1;%: log informations
options.nbitermax =100 ;%: log informatioouns
options.stopvarx=[0e0];% : threshold for stopping on F changes
options.stopvarj=[1e-8];
options.usegolden=1;
options.goldensearch_deltmax=.01;
options.goldensearch_stepmax=1;
options.numericalprecision=1e-3;

options.groups=[1:size(xapp,2)];%ones(10,1)%ones(10,1)%1:10
options.nu=2;
options.L0=1;
options.loss=0;

tic 
[glearn,obj,LOG]=lgrouplearn(xapp,yapp,options)
toc

[ypred] = svmval2(xtest,glearn);
ypred = reshape(ypred,nn,nn); 

figure(1)
plot(glearn.w, 'DisplayName', 'svm.w', 'YDataSource', 'svm.w'); figure(gcf)
figure(2)
plot(LOG.J)

%% biclasse validation

options.lambda = 1e-10;  
options.C = 3e3;
options.kernel='gaussian';
options.kerneloption=1;
options.verbose=0;
options.solver=3;

options.verboseloop=1;

options.nbloop=3;

options.params{1}='C';
options.values{1}=logspace(0,5,5);

options.params{2}='kerneloption';
options.values{2}=linspace(.1,50,5);

tic 
[svm,obj,LOG]=svmclass2valid(xapp,yapp,options)
toc

[ypred] = svmval2(xtest,svm);
ypred = reshape(ypred,nn,nn); 

%% test LARS SVM

options.lambda = 1e-5;  
options.kernel='linear';
options.kerneloption=1;
options.verbose=1;
options.multiclass=0;
options.nbselmax=[2:5];

tic 
[svm,obj]=larsvmclass(xapp,yapp,options)
toc

[ypred] = svmval2(xtest,svm);
ypred = reshape(ypred,nn,nn); 

%% GMKL

options.C = 1e2;
options.kerneltype=1;% 0:sum 1:prod
options.regtype=0; % 0:l1 1:l2
options.kerneloption=1;
options.verbose=1;

profile on

tic 
[gmkl,obj]=gmklclass(xapp,yapp,options)
toc

profile off
[ypred] = gmklval(xtest,gmkl);
ypred = reshape(ypred,nn,nn); 


%% multiclass

options.lambda = 1e-10;  
options.C = 3e3;
options.kernel='gaussian';
options.kerneloption=0.5;
options.verbose=1;
options.solver=4;
options.multiclass=1;
tic 
[svm,obj]=svmclass2(xapp,yapp,options)
toc

[ypred,ydec] = svmval2(xtest,svm);


test=get_precision(ytest,ypred)
%ypred = reshape(ydec,nn,nn); 

%% one classe

options.lambda = 1e-10;  
options.C = 1e-1;
options.nu=0.2;
options.kernel='gaussian';
options.kerneloption=0.4
options.verbose=1;
options.solver=1;

tic 
[svm,obj]=svmoneclass2(xapp,options)
toc

[ypred] = svmval2(xtest,svm);
ypred = reshape(ypred,nn,nn); 


%% preparation warm start

options=warmstart_svmclass2(xapp,options,svm)
%options.alphainit=zeros(length(xapp),1);4
%options.alphainit(svm.pos)=abs(svm.w);



%% linear
lambda = 1e-10;  
C = 1e5;


tic 
[w,w0,obj] = lsvmclass(xapp,yapp,C)
%[w,w0]=ldaclass(xapp,yapp,1/C)
toc
ypred = lsvmval(xtest,w,w0);%,framematrix,vector,dual)
ypred = reshape(ypred,nn,nn); 


figure(1)
plot(svm.w, 'DisplayName', 'svm.w', 'YDataSource', 'svm.w'); figure(gcf)
%%
%
%               plotting
%

figure(1); 
clf; 
 h1=plot(xapp(yapp==1,1),xapp(yapp==1,2),'+r'); hold on
h2=plot(xapp(yapp==-1,1),xapp(yapp==-1,2),'+b');
contour(xtest1,xtest2,ypred);%,[-1 0 1]

h3=plot(svm.xsup(:,1),svm.xsup(:,2),'ok'); 



%%
%
%               plotting multiclass
%

figure(1); 
clf; 
vals=sort(unique(yapp),'ascend');

contourf(xtest1,xtest2,ypred);hold on
h1=plot(xapp(yapp==1,1),xapp(yapp==1,2),'+r'); 
h2=plot(xapp(yapp==2,1),xapp(yapp==2,2),'+b'); 
h2=plot(xapp(yapp==3,1),xapp(yapp==3,2),'+g'); 
h2=plot(xapp(yapp==4,1),xapp(yapp==4,2),'+c'); 


%-------------------  Gaussian Kernel -------------------
