function [filtersvm,LOG]=adaptivescaling_svmclass2(x,y,options)
% function thta permits to learn an optimal scaling of features inside
% gaussian kernel
% classification
% Parameters:
% x : data
% y : classes( -1,1)
% options:
%  options.gamma : initialisation ...
%  options.kernel: kernel : gaussian of course
%  options.kerneloption: kernel option
%  options.lambda : lambda value for svm
%  options.C : C value for svm
%  options.verbose: print everything
%  options.log : log informations
%  options.nbitermax : log informations
%  options.stopcF : threshold for stopping on F changes
%  options.decy : y time shift...
%  options.solver : solver choice for non-linear svm
%
% Result:
% sigsvm:
%  sigsvm.kernel : kernel type in SVM-KM format
%                   'gaussian' or 'linear' leading to primal or dual
%                    optimization
%  sigsvm.xsup : support vector
%  sigsvm.w    : dual or primal variables weight
%  sigsvm.w0   : dual or primal zero value
%  sigsvm.pos  : position of support vectors (if non linear)
%  sigsvm.decy : time shift
%
%  Author : Remi Flamary remi (dot) flamary (at) gmail (dot) com



if ~isfield(options,'gamma'); options.gamma=ones(1,size(x,2)); end

if ~isfield(options,'kernel'); options.kernel='linear'; end
if ~isfield(options,'kerneloption'); options.kerneloption=1; end
if ~isfield(options,'lambda'); options.lambda=1e-10; end
if ~isfield(options,'C'); options.C=100; end
if ~isfield(options,'verbose'); options.verbose=0; end
if ~isfield(options,'log'); options.log=0; end
if ~isfield(options,'nbitermax'); options.nbitermax=50; end
if ~isfield(options,'decy'); options.decy=0; end
if ~isfield(options,'alphainit'); options.alphainit=[]; end
if ~isfield(options,'solver'); options.solver=0; end
if ~isfield(options,'numericalprecision'); options.numericalprecision=1e-15; end
if ~isfield(options,'use_cg'); options.use_cg=0; end
if ~isfield(options,'regnorm'); options.regnorm=1; end
if ~isfield(options,'pnorm'); options.pnorm=1; end
if ~isfield(options,'use_proj'); options.use_proj=0; end

if ~isfield(options,'multiclass'); options.multiclass=0; end

if length(options.gamma)==1
	options.gamma=options.gamma*ones(1,size(x,2))/size(x,2);
end

if length(unique(y))>2
    options.multiclass=1;
end

t0=cputime;

if options.multiclass==0
    % initialization

    LOG=[];
    
    gamma=options.gamma;

    %Xt=mfilter(F,X);

    % counter initialisation
    global nbcall
    nbcall=0;
    
    OPTIONS={'wbfixed','wfixed','lbfixed','lfixed','lupdate'};

    [Sigma,Xsup,Alpsup,w0,pos,Time,Crit,SigmaH] = svmfit(x,y,options.gamma,options.C,OPTIONS{options.solver},options.pnorm,options.verbose);


    gamma=Sigma;

    if options.log==1


        LOG.gamma=options.gamma;


        LOG.time=Time;
        LOG.J=Crit;
        LOG.gammatot=SigmaH;
        LOG.kernel=options.kernel;
        LOG.kerneloption=options.kerneloption;
        LOG.nbcostcall=nbcall;
    end

    % returned structure
    filtersvm.kernel=options.kernel;
    filtersvm.kerneloption=options.kerneloption;
    filtersvm.xsup =Xsup;
    filtersvm.w   =Alpsup;
    filtersvm.w0  =w0;
    filtersvm.pos =pos;
    filtersvm.gamma   =gamma;



else % multiclass

    vals=sort(unique(y),'ascend');

    if options.verbose~=0
        disp(['Multiclass SVM classification (' num2str(length(vals)) 'classes)' ]);
    end

    switch options.multiclass

        case 1 % one against all


            options2=options;
            options2.multiclass=0;
            
            for i=1:length(vals)

                if options.verbose~=0
                    disp(['Classification :' num2str(i) '/' num2str(length(vals)) ]);
                end;
                Yt=(y==vals(i))-(y~=vals(i));
                [models{i},LOG{i}]=adaptivescaling_svmclass(x,Yt,options2);

            end

            filtersvm.multiclass=options.multiclass;
            filtersvm.nbclass=length(vals);
            filtersvm.models=models;
            filtersvm.kerneloption=options.kerneloption;
            filtersvm.kernel=options.kernel;

    end




end

end

