function [filtersvm,LOG]=drsvmclass(x,y,options)
% function thta permits to learn an optimal progection matrix applied to x before
% classification
% Parameters:
% x : data
% y : classes( -1,1)
% options:
%  options.f : legth of the filter
%  options.P : initialisation of projection matrix
%  options.kernel: kernel
%  options.kerneloption: kernel option
%  options.lambda : lambda value for svm
%  options.C : C value for svm
%  options.verbose: print everything
%  option.dp : size of the projection subspace
%  options.log : log informations
%  options.nbitermax : log informations
%  options.stopcF : threshold for stopping on F changes
%  options.decy : y time shift...
%  options.solver : solver choice for non-linear svm
%
% Result:
% sigsvm:
%  sigsvm.kernel : kernel type in SVM-KM format
%                   'gaussian' or 'linear' leading to primal or dual
%                    optimization
%  sigsvm.xsup : support vector
%  sigsvm.w    : dual or primal variables weight
%  sigsvm.w0   : dual or primal zero value
%  sigsvm.pos  : position of support vectors (if non linear)
%  sigsvm.F    : optimal filter
%  sigsvm.decy : time shift
%
%  Author : Remi Flamary remi (dot) flamary (at) gmail (dot) com


if ~isfield(options,'dp'); options.dp=size(x,2); end
if ~isfield(options,'r'); options.r=0; end
%if ~isfield(options,'F'); options.F=ones(options.f,size(x,2)); end

if ~isfield(options,'kernel'); options.kernel='linear'; end
if ~isfield(options,'kerneloption'); options.kerneloption=1; end
if ~isfield(options,'lambda'); options.lambda=1e-10; end
if ~isfield(options,'C'); options.C=100; end
if ~isfield(options,'verbose'); options.verbose=0; end
if ~isfield(options,'log'); options.log=0; end
if ~isfield(options,'nbitermax'); options.nbitermax=50; end
if ~isfield(options,'decy'); options.decy=0; end
if ~isfield(options,'alphainit'); options.alphainit=[]; end
if ~isfield(options,'solver'); options.solver=0; end
if ~isfield(options,'numericalprecision'); options.numericalprecision=1e-15; end
if ~isfield(options,'use_cg'); options.use_cg=0; end
if ~isfield(options,'regnorm2'); options.regnorm2=1; end
if ~isfield(options,'regnorm12'); options.regnorm12=0; end
if ~isfield(options,'regnormdiff'); options.regnormdiff=0; end

if ~isfield(options,'multiclass'); options.multiclass=0; end
if ~isfield(options,'nbclass'); options.nbclass=length(unique(y)); end

if ~isfield(options,'sel'); options.sel=[]; end
if ~isfield(options,'selsparse'); options.selsparse=0; end

if length(unique(y))>2
    options.multiclass=1;
end

t0=cputime;


if ~isfield(options,'P'); 
    [PCcoeff, P] = pca(x, options.dp);
  %  P=randn(size(x,2), options.dp);
    options.P=P;
end


if options.multiclass==0
    % initialization
    P=options.P;
    dp=options.dp;

    if strcmp(options.kernel,'linear')
        linear=1;
    else
        linear=0;
    end
    LOG=[];
    

    %Xt=mfilter(F,X);


    % counter initialisation
    global nbcall
    nbcall=0;

    % data initialisation
    data=options;
    data.X=x;
    data.Y=y;
    data.options=options;


    % solver options initialisation
            
        opts=options;
        
        

    if linear==0
        opts.costfunc=@costfunc;
        opts.gradfunc=@gradfunc;
        opts.printfunc=@printfunc;
    else
        opts.costfunc=@costfunc_linear;
        opts.gradfunc=@gradfunc_linear;
    end
    
  


        % solving standard gradient descent problem problem
        [P,vars,costmin,LOG2]=grad_desc(P,[],data,opts);



    if options.log==1

        LOG.Ptot=LOG2.xtot;
        LOG.P=options.P;
        LOG.decy=options.decy;
        LOG.J=LOG2.J;
        LOG.time=LOG2.time;
        LOG.xsup=LOG2.vars.xsup;
        LOG.pos=LOG2.vars.pos;
        LOG.w=LOG2.vars.w;
        LOG.w0=LOG2.vars.w0;
        LOG.kernel=options.kernel;
        LOG.kerneloption=options.kerneloption;
        LOG.nbcostcall=nbcall;
    end

    % returned structure
    filtersvm.kernel=options.kernel;
    filtersvm.kerneloption=options.kerneloption;
    filtersvm.xsup =vars.xsup;
    filtersvm.w   =vars.w;
    filtersvm.w0  =vars.w0;
    filtersvm.pos =vars.pos;
    filtersvm.P   =P;
    filtersvm.nbclass=options.nbclass;


else % multiclass

    vals=sort(unique(y),'ascend');

    if options.verbose~=0
        disp(['Multiclass SVM classification (' num2str(length(vals)) 'classes)' ]);
    end

    switch options.multiclass

        case 1 % one against all


            options2=options;
            options2.multiclass=0;
            options2.nbclass=length(vals);
            
            for i=1:length(vals)

                if options.verbose~=0
                    disp(['Classification :' num2str(i) '/' num2str(length(vals)) ]);
                end;
                Yt=(y==vals(i))-(y~=vals(i));
                [models{i},LOG{i}]=filtersvmclass(x,Yt,options2);

            end

            filtersvm.multiclass=options.multiclass;
            filtersvm.nbclass=length(vals);
            filtersvm.models=models;
            filtersvm.kerneloption=options.kerneloption;
            filtersvm.kernel=options.kernel;

    end




end

end


function [costnew,varsnew]=costfunc(C,data,vars)
% cost function for the non-linear case

%cost counter
global nbcall
nbcall=nbcall+1;

varsnew=vars;

% filtering data
Xf=data.X*C;

% initialize alphas
alphainit=zeros(size(data.Y));
if ~isempty(vars)&&~isempty(vars.pos)
    alphainit(vars.pos)=abs(vars.w);
end
%data.alphainit=alphainit;
options=SetFields([],data,{'kernel','kerneloption','C','lambda','solver'});
options.verbose=0;
options.alphainit=alphainit;


% solve with warm start
[svm,obj]=svmclass2(Xf,data.Y,options);


%extract suport vectors
%xtemp=xsup;
%Ktemp=svmkernel(xtemp,data.kernel,data.kerneloption);

% cost calculation
costnew=obj+0.5*Omega2(C,data.regnorm2);

varsnew.pos=svm.pos;
varsnew.w=svm.w;
varsnew.w0=svm.w0;
varsnew.xsup=svm.xsup;

end

function [gradf]=gradfunc(C,data,vars0)
% gradient function for the non-linear case

    alpha=zeros(length(data.Y),1);
    alpha(vars0.pos)=abs(vars0.w);
    gradf=gradC_gaussian(data.X,alpha.*data.Y,C,data)+gradOmega2(C,data.regnorm2);

   
end

% 
% 
% function [costnew,varsnew]=costfunc_linear(F,data,vars)
% % cost function for the non-linear case
% 
% %cost counter
% global nbcall
% nbcall=nbcall+1;
% 
% varsnew=vars;
% pos=[];xsup=[];
% 
% % filtering data
% Xf=X*F;
% 
% %opt.lin_cg=1;
% %solving linear system
% [w,w0,obj]=lsvmclass(Xf,data.Y,data.C);
% 
% % linear cost
% costnew=1/2*sum(w.*w)+data.C*1/2*sum(max(0,1-data.Y.*(Xf*w+w0)))+1/2*Omega2(F,data.regnorm2)+data.regnorm12*Omega12(F)+data.regnormdiff*Omegadiff(F);
% 
% %costnew=obj+1/2*Omega2(F,data.regnorm2)+data.regnorm12*Omega12(F);
% 
% 
% varsnew.pos=pos;
% varsnew.w=w;
% varsnew.w0=w0;
% varsnew.xsup=xsup;
% 
% end
% 
% function [gradf]=gradfunc_linear(F,data,vars0)
% % gradient function for the non-linear case
% 
% gradf=data.C*lgradF(data.X,data.Y,vars0.w,vars0.w0,F)+gradOmega2(F,data.regnorm2)+data.regnormdiff*gradOmegadiff(F);
% 
% if data.r~=0
%     gradf(data.r+1:end,1:data.nbclass)=0;
% end
% 
% end





function [nbverbose]=printfunc(nbverbose,verbose,J,deltaJ,vars)

if isempty(vars)
    vars.w=[];
end

nbverbose = nbverbose+1;
if nbverbose == 20 || (nbverbose==1 && deltaJ==0);
    disp([repmat('    ',1,verbose-1) '      Cost     Delta Cost  nbsup']);
    nbverbose = 0;
end


fprintf(1,[repmat('    ',1,verbose-1) '| %11.4e | %8.4f | %d |\n'],[J deltaJ length(vars.w)]);

end




