function [filtersvm,LOG]=filtersvmclass(x,y,options)
% function thta permits to learn an optimal filter applied to x before
% classification
% Parameters:
% x : data
% y : classes( -1,1)
% options:
%  options.f : legth of the filter
%  options.F : initialisation of the filter
%  options.kernel: kernel
%  options.kerneloption: kernel option
%  options.lambda : lambda value for svm
%  options.C : C value for svm
%  options.verbose: print everything
%  options.log : log informations
%  options.nbitermax : log informations
%  options.stopcF : threshold for stopping on F changes
%  options.decy : y time shift...
%  options.solver : solver choice for non-linear svm
%
% Result:
% sigsvm:
%  sigsvm.kernel : kernel type in SVM-KM format
%                   'gaussian' or 'linear' leading to primal or dual
%                    optimization
%  sigsvm.xsup : support vector
%  sigsvm.w    : dual or primal variables weight
%  sigsvm.w0   : dual or primal zero value
%  sigsvm.pos  : position of support vectors (if non linear)
%  sigsvm.F    : optimal filter
%  sigsvm.decy : time shift
%
%  Author : Remi Flamary remi (dot) flamary (at) gmail (dot) com


if ~isfield(options,'f'); options.f=10; end
if ~isfield(options,'r'); options.r=0; end
if ~isfield(options,'F'); options.F=ones(options.f,size(x,2)); end

if ~isfield(options,'kernel'); options.kernel='linear'; end
if ~isfield(options,'kerneloption'); options.kerneloption=1; end
if ~isfield(options,'lambda'); options.lambda=1e-10; end
if ~isfield(options,'C'); options.C=100; end
if ~isfield(options,'verbose'); options.verbose=0; end
if ~isfield(options,'log'); options.log=0; end
if ~isfield(options,'nbitermax'); options.nbitermax=50; end
if ~isfield(options,'decy'); options.decy=0; end
if ~isfield(options,'alphainit'); options.alphainit=[]; end
if ~isfield(options,'solver'); options.solver=0; end
if ~isfield(options,'numericalprecision'); options.numericalprecision=1e-15; end
if ~isfield(options,'use_cg'); options.use_cg=0; end
if ~isfield(options,'regnorm2'); options.regnorm2=1; end
if ~isfield(options,'regnorm12'); options.regnorm12=0; end
if ~isfield(options,'regnormdiff'); options.regnormdiff=0; end

if ~isfield(options,'multiclass'); options.multiclass=0; end
if ~isfield(options,'nbclass'); options.nbclass=length(unique(y)); end

if ~isfield(options,'sel'); options.sel=[]; end
if ~isfield(options,'selsparse'); options.selsparse=0; end

if length(unique(y))>2
    options.multiclass=1;
end

t0=cputime;


    if options.r~=0
        X=[insertlabels(y,options.decy) x];
    else
        X=x;
    end


if options.multiclass==0
    % initialization
    F=options.F;
    f=options.f;

    if strcmp(options.kernel,'linear')
        linear=1;
    else
        linear=0;
    end
    LOG=[];
    
    Yt=DecalageSignal(y,options.decy);
    %Xt=mfilter(F,X);
    
    if options.r~=0
        F=[zeros(f,options.nbclass) F];    
        F(1,1:options.nbclass)=1;
    end

    % counter initialisation
    global nbcall
    nbcall=0;

    % data initialisation
    data=options;
    data.X=X;
    data.Y=Yt;
    data.options=options;


    % solver options initialisation
            
        opts=options;
        
        
    if isempty(options.sel)

    if linear==0
        opts.costfunc=@costfunc;
        opts.gradfunc=@gradfunc;
        opts.printfunc=@printfunc;
    else
        opts.costfunc=@costfunc_linear;
        opts.gradfunc=@gradfunc_linear;
    end
    
    else
        data.Y=Yt(logical(options.sel));
        
    if linear==0
        opts.costfunc=@costfuncsel;
        opts.gradfunc=@gradfuncsel;
        opts.printfunc=@printfunc;
    else
        opts.costfunc=@costfuncsel_linear;
        opts.gradfunc=@gradfuncsel_linear;
    end   
        
    end

    % solving
    if options.regnorm12==0
        % solving standard gradient descent problem problem
        [F,vars,costmin,LOG2]=grad_desc(F,[],data,opts);
    else
        % solving reweighted
        if ~isempty(options.sel)
            data.Y=Yt;
        end
        opts2=opts;
        opts2.updatefunc=@reweightedupdate;
        [F,vars,costmin,LOG2]=iterative_solver(F,[],data,opts2);

    end


    if options.log==1

        LOG.Ftot=LOG2.xtot;
        LOG.F=options.F;
        LOG.decy=options.decy;
        LOG.J=LOG2.J;
        LOG.time=LOG2.time;
        LOG.xsup=LOG2.vars.xsup;
        LOG.pos=LOG2.vars.pos;
        LOG.w=LOG2.vars.w;
        LOG.w0=LOG2.vars.w0;
        LOG.kernel=options.kernel;
        LOG.kerneloption=options.kerneloption;
        LOG.nbcostcall=nbcall;
    end

    % returned structure
    filtersvm.kernel=options.kernel;
    filtersvm.kerneloption=options.kerneloption;
    filtersvm.xsup =vars.xsup;
    filtersvm.w   =vars.w;
    filtersvm.w0  =vars.w0;
    filtersvm.pos =vars.pos;
    filtersvm.F   =F;
    filtersvm.decy=options.decy;
    filtersvm.r=options.r;
    filtersvm.nbclass=options.nbclass;
    filtersvm.t=options.t;


else % multiclass

    vals=sort(unique(y),'ascend');

    if options.verbose~=0
        disp(['Multiclass SVM classification (' num2str(length(vals)) 'classes)' ]);
    end

    switch options.multiclass

        case 1 % one against all


            options2=options;
            options2.multiclass=0;
            options2.nbclass=length(vals);
            
            for i=1:length(vals)

                if options.verbose~=0
                    disp(['Classification :' num2str(i) '/' num2str(length(vals)) ]);
                end;
                Yt=(y==vals(i))-(y~=vals(i));
                [models{i},LOG{i}]=filtersvmclass(x,Yt,options2);

            end

            filtersvm.multiclass=options.multiclass;
            filtersvm.nbclass=length(vals);
            filtersvm.models=models;
            filtersvm.kerneloption=options.kerneloption;
            filtersvm.kernel=options.kernel;

    end




end

end


function [costnew,varsnew]=costfunc(F,data,vars)
% cost function for the non-linear case

%cost counter
global nbcall
nbcall=nbcall+1;

varsnew=vars;

% filtering data
Xf=mfilter(F,data.X);

% initialize alphas
alphainit=zeros(size(data.Y));
if ~isempty(vars)&&~isempty(vars.pos)
    alphainit(vars.pos)=abs(vars.w);
end
%data.alphainit=alphainit;
options=SetFields([],data,{'kernel','kerneloption','C','lambda','solver'});
options.verbose=0;
options.alphainit=alphainit;


% solve with warm start
[svm,obj]=svmclass2(Xf,data.Y,options);


%extract suport vectors
%xtemp=xsup;
%Ktemp=svmkernel(xtemp,data.kernel,data.kerneloption);

% cost calculation
costnew=obj+0.5*Omega2(F,data.regnorm2)+data.regnorm12*Omega12(F)+data.regnormdiff*Omegadiff(F);

varsnew.pos=svm.pos;
varsnew.w=svm.w;
varsnew.w0=svm.w0;
varsnew.xsup=svm.xsup;

end

function [gradf]=gradfunc(F,data,vars0)
% gradient function for the non-linear case

    alpha=zeros(length(data.Y),1);
    alpha(vars0.pos)=abs(vars0.w);
    gradf=gradF_gaussian(data.X,alpha.*data.Y,F,data)+gradOmega2(F,data.regnorm2)+data.regnormdiff*gradOmegadiff(F);

   


if data.r~=0
    gradf(data.r+1:end,1:data.nbclass)=0;
end

end



function [costnew,varsnew]=costfunc_linear(F,data,vars)
% cost function for the non-linear case

%cost counter
global nbcall
nbcall=nbcall+1;

varsnew=vars;
pos=[];xsup=[];

% filtering data
Xpsd=get_F_PSD(F,data.X,data.t);

%opt.lin_cg=1;
%solving linear system
[w,w0,obj]=lsvmclass(Xpsd,data.Y,data.C);

% linear cost
costnew=1/2*sum(w.*w)+data.C*1/2*sum(max(0,1-data.Y.*(Xpsd*w+w0)))+1/2*Omega2(F,data.regnorm2)+data.regnorm12*Omega12(F)+data.regnormdiff*Omegadiff(F);

%costnew=obj+1/2*Omega2(F,data.regnorm2)+data.regnorm12*Omega12(F);


varsnew.pos=pos;
varsnew.w=w;
varsnew.w0=w0;
varsnew.xsup=xsup;

end

function [gradf]=gradfunc_linear(F,data,vars0)
% gradient function for the non-linear case

gradf=data.C*lgradFpsd(data.X,data.Y,vars0.w,vars0.w0,F,data.t)+gradOmega2(F,data.regnorm2)+data.regnormdiff*gradOmegadiff(F);

end


function [costnew,varsnew]=costfuncsel_linear(F,data,vars)
% cost function for the non-linear case

%cost counter
global nbcall
nbcall=nbcall+1;

varsnew=vars;
pos=[];xsup=[];

% filtering data
%tic
%Xf=mfilter(F,data.X);
%Xf=Xf(logical(data.sel),:);

if data.selsparse~=0
    Xf=GetFiltTimeSamples(F,data.X,data.sel);
else
    Xf=mfilter(F,data.X);
    Xf=Xf(logical(data.sel),:);
end
%toc


%opt.lin_cg=1;
%solving linear system
[w,w0,obj]=lsvmclass(Xf,data.Y,data.C);

% linear cost
costnew=1/2*sum(w.*w)+data.C*1/2*sum(max(0,1-data.Y.*(Xf*w+w0)))+1/2*Omega2(F,data.regnorm2)+data.regnorm12*Omega12(F)+data.regnormdiff*Omegadiff(F);

%costnew=obj+1/2*Omega2(F,data.regnorm2)+data.regnorm12*Omega12(F);


varsnew.pos=pos;
varsnew.w=w;
varsnew.w0=w0;
varsnew.xsup=xsup;

end

function [gradf]=gradfuncsel_linear(F,data,vars0)
% gradient function for the non-linear case

gradf=data.C*lgradFsel(data.X,data.Y,vars0.w,vars0.w0,F,data.sel,data.selsparse)+gradOmega2(F,data.regnorm2)+data.regnormdiff*gradOmegadiff(F);

if data.r~=0
    gradf(data.r+1:end,1:data.nbclass)=0;
end

end

function [costnew,varsnew]=costfuncsel(F,data,vars)
% cost function for the non-linear case

%cost counter
global nbcall
nbcall=nbcall+1;

varsnew=vars;

% filtering data

if data.selsparse~=0
    Xf=GetFiltTimeSamples(F,data.X,data.sel);
else
    Xf=mfilter(F,data.X);
    Xf=Xf(logical(data.sel),:);
end

% initialize alphas
alphainit=zeros(size(data.Y));
if ~isempty(vars)&&~isempty(vars.pos)
    alphainit(vars.pos)=abs(vars.w);
end
%data.alphainit=alphainit;
options=SetFields([],data,{'kernel','kerneloption','C','lambda','solver'});
options.verbose=0;
options.alphainit=alphainit;


% solve with warm start
[svm,obj]=svmclass2(Xf,data.Y,options);


%extract suport vectors
%xtemp=xsup;
%Ktemp=svmkernel(xtemp,data.kernel,data.kerneloption);

% cost calculation
costnew=obj+0.5*Omega2(F,data.regnorm2)+data.regnorm12*Omega12(F)+data.regnormdiff*Omegadiff(F);

varsnew.pos=svm.pos;
varsnew.w=svm.w;
varsnew.w0=svm.w0;
varsnew.xsup=svm.xsup;

end

function [gradf]=gradfuncsel(F,data,vars0)
% gradient function for the non-linear case

    alpha=zeros(length(data.Y),1);
    alpha(vars0.pos)=abs(vars0.w);
    gradf=gradF_gaussiansel(data.X,alpha.*data.Y,F,data)+gradOmega2(F,data.regnorm2)+data.regnormdiff*gradOmegadiff(F);

   


if data.r~=0
    gradf(data.r+1:end,1:data.nbclass)=0;
end

end


function [nbverbose]=printfunc(nbverbose,verbose,J,deltaJ,vars)

if isempty(vars)
    vars.w=[];
end

nbverbose = nbverbose+1;
if nbverbose == 20 || (nbverbose==1 && deltaJ==0);
    disp([repmat('    ',1,verbose-1) '      Cost     Delta Cost  nbsup']);
    nbverbose = 0;
end


fprintf(1,[repmat('    ',1,verbose-1) '| %11.4e | %8.4f | %d |\n'],[J deltaJ length(vars.w)]);

end

function [F,costnew,varsnew]=reweightedupdate(F_1,vars,data)
% update function for reweignted

if isempty(vars)
    % initialisation
    options=data.options;
    regnorm2=options.regnorm12;
    options.regnorm12=0;
    options.regnorm2=regnorm2*ones(size(F_1,2),1);

    if options.verbose~=0
        options.verbose=options.verbose+1;
    end
    vars.options=options;
    vars.regnorm=regnorm2;
    vars.options.decy=0;
    vars.LOG2=[];

end


vars.options.F=F_1;

% solve l2 problem
[filtersvm,LOG]=filtersvmclass(data.X,data.Y,vars.options);

F=filtersvm.F;

if strcmp(data.options.kernel,'linear')
    costnew=costfunc_linear(F,data,filtersvm);
else
    costnew=costfunc(F,data,filtersvm);
end

% reweighting!!
vars.options.F=F;
wgth=sqrt(sum(F.*F));
vars.options.regnorm2=  vars.regnorm./wgth;
vars.w=filtersvm.w;
vars.w0=filtersvm.w0;
vars.pos=filtersvm.pos;
vars.xsup=filtersvm.xsup;


varsnew=vars;

if vars.options.log~=0
    varsnew.LOG2{length(varsnew.LOG2)+1}=LOG;
end



end




