%lassification 
%   Comparing Gaussian and polynomial Kernels 
%
%


clear all
close all

%%
%
%   Creating Data
%
seed=1985
rand('state',seed);
randn('state',seed);

nbapp=1000;
nn=20;
nbtest=nn*nn;
sigma=1;
[xapp,yapp,xtest,ytest,xtest1,xtest2]=datasets('clowns',nbapp,nbtest,sigma);
%------- Building a 2D Grid for function evaluation -------------------------
[xtest1 xtest2]  = meshgrid([-1:.05:1]*3.5,[-1:0.05:1]*3); 
nn = length(xtest1); 
xtest = [reshape(xtest1 ,nn*nn,1) reshape(xtest2 ,nn*nn,1)]; 

idxs=randperm(length(yapp));

yapp=yapp(idxs);
xapp=xapp(idxs,:);

nbdim=10;
P=randn(2,nbdim);

xapp=xapp*P;
xtest=xtest*P;

nbnoise=10;
pbruit=1;
xapp=[xapp+pbruit*randn(size(xapp))];
xtest=[xtest+pbruit*zeros(size(xtest))];

%
%  learning with polynomial kernels
%





%% biclasse

options.lambda = 1e-10;  
options.C = 1e4;
options.kernel='gaussian';
options.kerneloption=1;
options.verbose=0;
options.solver=4;
options.multiclass=0;

tic 
[svm,obj]=svmclass2(xapp,yapp,options)
toc

[ypred] = svmval2(xtest,svm);
ypred = reshape(ypred,nn,nn); 

%plot(svm.w, 'DisplayName', 'svm.w', 'YDataSource', 'svm.w'); figure(gcf)


%% test dr_svm

dp=2;
options.dp=dp; % : legth of the filter


options.verbose= 1;%: print everything
options.log =1;%: log informations
options.nbitermax =40 ;%: log informatioouns
options.stopvarx=[0];% : threshold for stopping on F changes
options.stopvarj=[1e-4];
options.goldensearch_deltmax=.001;
options.goldensearch_stepmax=1;
options.numericalprecision=1e-3;
options.multiclass=0;
options.linesearch=0;
options.ls_tol=1e-3;
options.testgrad=0.000001;



linear=0;
if linear
    options.kernel='linear';
    options.kerneloption=1;
    options.C = 1e3; %: C value for svm
    options.regnorm2=1e4;%[2.25e-2];
    options.regnorm12=0e-3;%[2.25e-2];
    options.regnormdiff=0e4;
    options.solver=4;
    options.use_cg=1;
else
    options.kernel='gaussian';
    options.kerneloption=sqrt(size(xapp,2)/2);
    options.C = 1e3; %: C value for svm
    options.regnorm2=1e0;%[2.25e-2];
    options.regnormdiff=0e3;
    options.regnorm12=0e2;%[2.25e-2];
    options.solver=4;
    options.use_cg=1;
end

tic 
[drsvm,obj]=drsvmclass(xapp,yapp,options)
toc


[ypred] = drsvmval(xtest,drsvm);
ypred = reshape(ypred,nn,nn);

%%
%
%               plotting
%

figure(1); 
clf; 
 h1=plot(xapp(yapp==1,1),xapp(yapp==1,2),'+r'); hold on
h2=plot(xapp(yapp==-1,1),xapp(yapp==-1,2),'+b');
contour(xtest1,xtest2,ypred);%,[-1 0 1]

h3=plot(svm.xsup(:,1),svm.xsup(:,2),'ok'); 

%% plotpca

[PCcoeff, PCvec] = pca(xapp, nbdim);

xpca=xapp*PCvec;

figure(1); 
clf; 
 h1=plot(xpca(yapp==1,1),xpca(yapp==1,2),'+r'); hold on
h2=plot(xpca(yapp==-1,1),xpca(yapp==-1,2),'+b');

xpca=xapp*drsvm.P;

figure(2); 
clf; 
 h1=plot(xpca(yapp==1,1),xpca(yapp==1,2),'+r'); hold on
h2=plot(xpca(yapp==-1,1),xpca(yapp==-1,2),'+b');
%%
%
%               plotting multiclass
%

figure(1); 
clf; 
vals=sort(unique(yapp),'ascend');

contourf(xtest1,xtest2,ypred);hold on
h1=plot(xapp(yapp==1,1),xapp(yapp==1,2),'+r'); 
h2=plot(xapp(yapp==2,1),xapp(yapp==2,2),'+b'); 
h2=plot(xapp(yapp==3,1),xapp(yapp==3,2),'+g'); 
h2=plot(xapp(yapp==4,1),xapp(yapp==4,2),'+c'); 


%-------------------  Gaussian Kernel -------------------
